#include "Utils.h"
#include "Animation.h"
#include "Model.h"
#include "objreader.h"

Animation::Animation(Model* target, bool repeat) {
    this->target = target;
    this->repeat = repeat;
    active = false;
    stopGraceful = false;
    numFrames = 0;
    currentFrame = 0;
    currentFrameTimeLeft = 0;
}

void Animation::setModelToFrame(int i) {
    Frame* frame = frames[i];
    coord* frameCoords = frame->getFrameVertices();
    coord* modelCoords = target->getModelVertices();
    int numVertices = target->getModelNumVertices();
    for (int i = 0; i < numVertices ; i++) {
        modelCoords[i].x = frameCoords[i].x;
        modelCoords[i].y = frameCoords[i].y;
        modelCoords[i].z = frameCoords[i].z;    
    }
}

void Animation::interpolateModelToFrame(float deltaTime, float timeLeft) {
    if (timeLeft > 0.005) {
        Frame* frame = frames[currentFrame];
        coord* frameCoords = frame->getFrameVertices();
        coord* modelCoords = target->getModelVertices();
        int numVertices = target->getModelNumVertices();
        for (int i = 0; i < numVertices ; i++) {
            modelCoords[i].x = Utils::interpVal(frameCoords[i].x, modelCoords[i].x, deltaTime, timeLeft);
            modelCoords[i].y = Utils::interpVal(frameCoords[i].y, modelCoords[i].y, deltaTime, timeLeft);
            modelCoords[i].z = Utils::interpVal(frameCoords[i].z, modelCoords[i].z, deltaTime, timeLeft);   
        }
    }
}

void Animation::addFrame(Frame* frame) {
    frames.push_back(frame);
    numFrames++;    
}

int Animation::getNumFrames() {
    return numFrames;
}
    
void Animation::reset() {
    currentFrame = 0;
    if (numFrames > 0) {
        currentFrameTimeLeft = frames[currentFrame]->getFrameLength(); 
        active = true; 
        stopGraceful = false;
    }
}

bool Animation::isActive() {
    return active;   
}

void Animation::stopGracefully() {
    stopGraceful = true;
}
    
void Animation::animate(float deltaTime) {
    if (active) {
        if (currentFrameTimeLeft > 0.005) {
            interpolateModelToFrame(deltaTime, currentFrameTimeLeft);
            currentFrameTimeLeft -= deltaTime;
        } else {
            if (currentFrame == numFrames-1) {
                if (!repeat || stopGraceful) {
                    active = false;   
                }
                currentFrame = 0;
                currentFrameTimeLeft = frames[0]->getFrameLength();                       
            } else {
                currentFrame++;
                currentFrameTimeLeft = frames[currentFrame]->getFrameLength(); 
            }           
        }
    }
}
