#include "Beast.h"
#include <math.h>
#include "Room.h"

Beast::Beast(Detector* detector, Wizard* wizard, Room* room, int level) : Enemy(detector, wizard, room, level, "Models/beastie1.obj", "Textures/beastie1.tga") {
    frame0 = new Frame("Models/beastie1.obj", 0.2);
    frame1 = new Frame("Models/beastie1-walk1.obj", 0.2);
    frame2 = new Frame("Models/beastie1-walk2.obj", 0.2);
    walkAnimation = new Animation(model, true);
    idleAnimation = new Animation(model, false);
    walkAnimation->addFrame(frame1);
    walkAnimation->addFrame(frame0);
    walkAnimation->addFrame(frame2);
    walkAnimation->addFrame(frame0);
    idleAnimation->addFrame(frame0);
    model->addAnimation(BEAST_WALK, walkAnimation);
    model->addAnimation(BEAST_REST, idleAnimation);
    model->setCurrentAnimation(BEAST_REST);    
}

Beast::~Beast() {
    delete walkAnimation;
    delete idleAnimation;
    delete frame0;
    delete frame1;
    delete frame2;
}

void Beast::moveForward(float deltaTime) {
    Object::moveForward(level*deltaTime);
    if (model->isAnimating() && (model->getCurrentAnimation() != BEAST_WALK)) {
        model->setCurrentAnimation(BEAST_WALK);
    } else if (!model->isAnimating()) {
        model->setCurrentAnimation(BEAST_WALK);
        model->startAnimation();
    }
}

void Beast::stop() {
    model->setCurrentAnimation(BEAST_REST);
    if (!model->isAnimating()) {
        model->startAnimation();    
    }
}
