#include "BillboardParticle.h"

BillboardParticle::BillboardParticle(float lifetime, GLfloat x, GLfloat y, GLfloat z, 
                      GLfloat r, GLfloat g, GLfloat b, GLfloat a,
                      GLuint textureId, float size) : Particle(x, y, z) {
    this->lifetime = lifetime;
    color[0] = r;
    color[1] = g;
    color[2] = b;
    color[3] = a;   
    this->textureId = textureId;
    this->size = size;              
}

void BillboardParticle::draw() {
    
    // http://nehe.gamedev.net/data/articles/article.asp?article=19
    // http://www.lighthouse3d.com/opengl/billboarding/index.php3?billCheat2
    
    GLfloat right[3];
    GLfloat up[3];
    
    GLfloat modelview[16];
    glGetFloatv(GL_MODELVIEW_MATRIX, (GLfloat*)modelview);
    
    right[0] = modelview[0];
    right[1] = modelview[4];
    right[2] = modelview[8];
    
    up[0] = modelview[1];
    up[1] = modelview[5];
    up[2] = modelview[9];
    
    GLfloat v0[3];
    GLfloat v1[3];
    GLfloat v2[3];
    GLfloat v3[3];
    
    
    /*   v3-------v2
     *   |        |
     *   |   pos  |
     *   |        |
     *   v0-------v1
     */
     
    v0[0] = position[0] - (right[0] + up[0]) * size;
    v0[1] = position[1] - (right[1] + up[1]) * size;
    v0[2] = position[2] - (right[2] + up[2]) * size;
    
    v1[0] = position[0] + (right[0] - up[0]) * size;
    v1[1] = position[1] + (right[1] - up[1]) * size;
    v1[2] = position[2] + (right[2] - up[2]) * size;

    v2[0] = position[0] + (right[0] + up[0]) * size;
    v2[1] = position[1] + (right[1] + up[1]) * size;
    v2[2] = position[2] + (right[2] + up[2]) * size;   
    
    v3[0] = position[0] - (right[0] - up[0]) * size;
    v3[1] = position[1] - (right[1] - up[1]) * size;
    v3[2] = position[2] - (right[2] - up[2]) * size;     
    
    glColor4fv(color);
    glBindTexture(GL_TEXTURE_2D, textureId);
    glBegin(GL_TRIANGLE_FAN);
        glTexCoord2f(0,0);
        glVertex3fv(v0);
        glTexCoord2f(1,0);
        glVertex3fv(v1);
        glTexCoord2f(1,1);
        glVertex3fv(v2);
        glTexCoord2f(0,1);
        glVertex3fv(v3);                
    glEnd();
    glBindTexture(GL_TEXTURE_2D, 0);
}

void BillboardParticle::idle(float deltaTime) {
    lifetime -= deltaTime;
    if (lifetime <= 0) {
        alive = false;    
    }
}
