#include "Detector.h"
#include <math.h>
#include "Object.h"
#include "Wall.h"
#include "Wizard.h"
#include "Beast.h"
#include <typeinfo>
#include "Gold.h"
#include "Spray.h"
#include "Spider.h"
#include "Enemy.h"

Detector::Detector() {
    
}

Object* Detector::collisionDetect(Object* object, float x, float y, float z) {
    std::list<Object*>::iterator i;
    
    float tx0,tx1,ty0,ty1,tz0,tz1;
    object->getBoundingVolume(&tx0,&tx1,&ty0,&ty1,&tz0,&tz1);
    tx0 += x;
    tx1 += x;
    ty0 += y;
    ty1 += y;
    tz0 += z;
    tz1 += z;
    
    for (i = objects.begin(); i != objects.end(); i++) {
        Object* other = *i;
        if (other != object) {
            float x0,x1,y0,y1,z0,z1;
            other->getBoundingVolume(&x0,&x1,&y0,&y1,&z0,&z1);
            x0 += other->getPositionX();
            x1 += other->getPositionX();
            y0 += other->getPositionY();
            y1 += other->getPositionY();
            z0 += other->getPositionZ();
            z1 += other->getPositionZ();
            
            if (((x0 <= tx0 && tx0 <= x1) || (x0 <= tx1 && tx1 <= x1) || (x0 <= tx0 && x1 >= tx1) || (x0 >= tx0 && x1 <= tx1)) && 
                ((y0 <= ty0 && ty0 <= y1) || (y0 <= ty1 && ty1 <= y1) || (y0 <= ty0 && y1 >= ty1) || (y0 >= ty0 && y1 <= ty1)) &&
                ((z0 <= tz0 && tz0 <= z1) || (z0 <= tz1 && tz1 <= z1) || (z0 <= tz0 && z1 >= tz1) || (z0 >= tz0 && z1 <= tz1))) {
                if (typeid(*object) == typeid(Wizard) && typeid(*other) == typeid(Gold)) {
                    ((Wizard*) object)->pickup( (Gold*) other);
                }
                if ((typeid(*object) == typeid(Beast) || typeid(*object) == typeid(Spider)) && typeid(*other) == typeid(Spray)) {
                    ((Enemy*) object)->spray(((Spray*) other)->getStrength());
                }
                if (typeid(*object) == typeid(Fireball)) {
                    ((Fireball*) object)->markForDeletion(true);
                    if (typeid(*other) == typeid(Beast) || typeid(*other) == typeid(Spider)) {
                        ((Enemy*) other)->changeHealth(-((Fireball*) object)->getStrength());
                    }
                }
                return other;
            }
        }
    }
    return NULL;
}

void Detector::idle(float deltaTime) {
    std::list<Object*>::iterator i;
    Wizard* wizard;
    for (i = objects.begin(); i != objects.end(); i++) {
        if (typeid(**i) == typeid(Wizard)) {
            wizard = (Wizard*) *i;
            break;
        }
    }
    float tx0,tx1,ty0,ty1,tz0,tz1;
    wizard->getBoundingVolume(&tx0,&tx1,&ty0,&ty1,&tz0,&tz1);
    tx0 += wizard->getPositionX();
    tx1 += wizard->getPositionX();
    ty0 += wizard->getPositionY();
    ty1 += wizard->getPositionY();
    tz0 += wizard->getPositionZ();
    tz1 += wizard->getPositionZ();
    for (i = objects.begin(); i != objects.end(); i++) {
        if (typeid(**i) == typeid(Beast) || typeid(**i) == typeid(Spider)) {
            Enemy* beast = (Enemy*) *i;
                if (beast->isAlive()) {            
                float x0,x1,y0,y1,z0,z1;
                beast->getBoundingVolume(&x0,&x1,&y0,&y1,&z0,&z1);
                x0 += beast->getPositionX()-0.3;
                x1 += beast->getPositionX()+0.3;
                y0 += beast->getPositionY()-0.3;
                y1 += beast->getPositionY()+0.3;
                z0 += beast->getPositionZ()-0.3;
                z1 += beast->getPositionZ()+0.3;
                
                if (((x0 <= tx0 && tx0 <= x1) || (x0 <= tx1 && tx1 <= x1) || (x0 <= tx0 && x1 >= tx1) || (x0 >= tx0 && x1 <= tx1)) && 
                    ((y0 <= ty0 && ty0 <= y1) || (y0 <= ty1 && ty1 <= y1) || (y0 <= ty0 && y1 >= ty1) || (y0 >= ty0 && y1 <= ty1)) &&
                    ((z0 <= tz0 && tz0 <= z1) || (z0 <= tz1 && tz1 <= z1) || (z0 <= tz0 && z1 >= tz1) || (z0 >= tz0 && z1 <= tz1))) {
                    beast->attack(wizard);
                }
            }
        }
    }
        
}

Enemy* Detector::findEnemyByNum(int num) {
    std::list<Object*>::iterator i;
    Enemy* beast;
    for (i = objects.begin(); i != objects.end(); i++) {
        if (typeid(**i) == typeid(Beast) || typeid(**i) == typeid(Spider)) {
            beast = (Enemy*) *i;
            if (beast->getId() == num) {
                return beast;  
            }
        }
    }
    return NULL;    
}

float Detector::dist(float x0, float y0, float z0, float x1, float y1, float z1) {
    float dx = x1 - x0;
    float dy = y1 - y0;
    float dz = z1 - z0;
    return sqrt(dx*dx + dy*dy + dz*dz);
}

void Detector::addObject(Object* object) {
    objects.push_back(object);
}

void Detector::removeObject(Object* object) {
        objects.remove(object);
}
