#include "Enemy.h"
#include <math.h>
#include "Room.h"
#include <stdio.h>

int Enemy::idCounter = 0;

Enemy::Enemy(Detector* detector, Wizard* wizard, Room* room, int level, std::string obj, std::string texture) : Object(detector, obj, texture) {
    model->setHeadingCorrection(270);
    model->setScale(0.5);
    this->wizard = wizard;
    this->room = room;
    enemyId = idCounter++;
    
    lastAttackTime = 0;
    this->level = level;
    melter = new MeltingModelParticleSystem(model, 1,1,0,0);
    health = 10 * level;
    maxHealth = 10 * level;
    dead = false;
    lastSprayTime = 0;

}

Enemy::~Enemy() {
    detector->removeObject(this);
    room->removeEnemy(this);
    delete melter;
    
}

bool Enemy::isAlive() {
    return !dead;
}

bool Enemy::isDeathAnimationDone() {   
    return !melter->isActive();
}

void Enemy::changeHealth(int amount) {
    if (health < maxHealth - amount) {
        health+= amount;
    } else {
        health = maxHealth;
    }
    
    if (amount < 0) {
        textColor[0] = 1;
        textColor[1] = 0;
        textColor[2] = 0;    
    } else {
        textColor[0] = 0;
        textColor[1] = 1;
        textColor[2] = 0;  
    }
    
    char msg[10];
    sprintf(msg, "%d", amount);
    showText(msg, 0.5);
    
    if (health <= 0) {
        if (!dead) {
            health = 0;
            dead = true;    
            melter->reset();
            melter->setActive(true);
            stop();
        }
    }

}

void Enemy::draw() {
    glPushMatrix();    
    glTranslatef(positionX,positionY,positionZ);
    glRotatef(orientation,0,1,0);
    if (dead) {
        melter->draw();
    } else {
        switch(level) {
            case 5:
            glColor3f(1,0.2,0.2);
            break;
            case 4:
            glColor3f(1,1,0);
            break;
            case 3:
            glColor3f(0.2,1,0.2);
            break;
            case 2:
            glColor3f(0.2,0.2,1.0);
            break;
            case 1:
            glColor3f(0.8,0.0,0.8);
            break;
            case 6: // for boss
            glColor3f(0.8,0.8,0.8);
            break;
        }
        glBindTexture(GL_TEXTURE_2D, tex);
        model->draw();
        if (bshowText) {
            glColor3f(textColor[0],textColor[1],textColor[2]);
            Utils::renderString(0,1.5,0, textToShow, GLUT_BITMAP_HELVETICA_18);    
        }
    }
    glPopMatrix();
}

void Enemy::idle(float deltaTime) {
    if (dead) {
        melter->idle(deltaTime);
    } else {
        Object::idle(deltaTime);
        if (calculateOrientation()) {;
            moveForward(deltaTime);
        } else {
            moveForward(0);
            stop();
        }
    }
}

bool Enemy::calculateOrientation() {
    float x = wizard->getPositionX();
    float y = wizard->getPositionY();
    float z = wizard->getPositionZ();
    if (room->inRoom(x,y,z)) {
        orientation = 90 + atan2f((x-positionX),(z-positionZ))*180.0/M_PI;
        return true;
    } else {
        return false;
    }
}

void Enemy::attack(Wizard* wizard) {
    int currentTime = glutGet(GLUT_ELAPSED_TIME);
    if (currentTime > lastAttackTime + 1000) {
        lastAttackTime = currentTime;
        wizard->changeHealth(-level*2);
    }
}

void Enemy::spray(int strength) {    
    int currentTime = glutGet(GLUT_ELAPSED_TIME);
    if (currentTime > lastSprayTime + 100) {
        lastSprayTime = currentTime;
        changeHealth(-strength);
    }
}
