#include "Fireball.h"
#include "Wizard.h"
#include <math.h>

Fireball::Fireball(Detector* detector, int power, Wizard* wizard) : Spell(detector, power) {
    this->wizard = wizard;
    particles = new FireballParticleSystem(100);
    particles->setActive(true);
    marked = false;
    strength = 1;
}

Fireball::~Fireball() {
    wizard->removeFireball(this);
    detector->removeObject(this);
    delete particles;
}
    
    
void Fireball::idle(float deltaTime) {
    computeStrength(deltaTime);
    if (!marked) {
        moveForward(10*deltaTime);
        //wizard->setAnimating(true);
    }
    particles->idle(deltaTime);
}

void Fireball::draw() {
    glPushMatrix();    
    glTranslatef(positionX,positionY,positionZ);
    glRotatef(orientation,0,1,0);
    if (particles->getAllowRenew()) {
        glEnable(GL_BLEND);
        glColor4f(1,0,0,0.5);
    if (strength < 1) {
        glutSolidSphere(0.2*strength, 8, 8);
    } else {
        glutSolidSphere(0.2, 8, 8);
    }
        glDisable(GL_BLEND);
    }
    particles->draw();
    glPopMatrix();
}

void Fireball::getBoundingVolume(float *minX,float *maxX,
                   float *minY,float *maxY,
                   float *minZ,float *maxZ) {
    *minX = -0.5;
    *maxX = 0.5;
    *minY = -0.5;
    *maxY = 0.5;
    *minZ = -0.5;
    *maxZ = 0.5;                
}

void Fireball::markForDeletion(bool explode) {
    marked = true;
    if (explode) {
        wizard->setAnimating(false);
        detector->removeObject(this);
        particles->explode();
    } else {
        particles->setActive(false);   
    }
}

bool Fireball::isMarked() {
    return marked && !particles->isActive();
}
int Fireball::getStrength() {
    return (int) round((power*2 + 3) * strength);
}

void Fireball::computeStrength(float deltaTime) {
    strength -= deltaTime/2.;
    if (strength < 0.1) {
        markForDeletion(false);
    }
}
