#ifndef MODEL_H
#define MODEL_H

#include <map>
#include <string>
#include "objreader.h"
#include "GL/glut.h"
#include "Animation.h"

class Model {

protected:
    GLfloat positionCorrection[3];
    GLfloat headingCorrection;
    std::map<int, Animation*> animations;
    
    bool animationActive;
    int currentAnimation;
    int numAnimations;
    
    objReader object;
    float scale;
    GLuint textureId;
    
public:
    
    Model();
    Model(std::string modelFileName, std::string textureFileName);
    
    void setPositionCorrection(float x, float y, float z);
    void setHeadingCorrection(float h);
    
    float getPositionCorrectionX() { return positionCorrection[0]; }
    float getPositionCorrectionY() { return positionCorrection[1]; }
    float getPositionCorrectionZ() { return positionCorrection[2]; }
    float getHeadingCorrection() { return headingCorrection; }
    
    virtual void setScale(float s);
    float getScale() { return scale; }
    
    void addAnimation(int animId, Animation* animation);
    void removeAnimation(int animId);
    void setCurrentAnimation(int animId);
    
    coord* getModelVertices();
    int getModelNumVertices();
    
    void startAnimation();
    void stopAnimation();
    
    bool isAnimating();
    int getCurrentAnimation();
    
    void idle(float deltaTime);
    virtual void draw();
    
    virtual void getBoundingVolume(float *minX,float *maxX,
                           float *minY,float *maxY,
                           float *minZ,float *maxZ);
};

#endif
