#include "Object.h"
#include "WallModel.h"
#include <iostream>
#include <math.h>
#define min(a,b) (((a)<(b))?(a):(b))
#define max(a,b) (((a)>(b))?(a):(b))

Object::Object() {
    model = NULL;
    positionX = positionY = positionZ = orientation = 0;
    bshowText = false;
    textTimeLeft = 0;
    memset(textToShow, '\0', 13); 
    textColor[1] = textColor[2] = textColor[3] = 1;
}

Object::~Object() {
    if (model != NULL) {
        delete model;
    }
}



Object::Object(Detector* detector, int wall_type) {
    model = new WallModel(wall_type);
    this->detector = detector;
    positionX = positionY = positionZ = orientation = 0;
    bshowText = false;
    textTimeLeft = 0;
    memset(textToShow, '\0', 13); 
    textColor[1] = textColor[2] = textColor[3] = 1;
    
    detector->addObject(this);
}

Object::Object(Detector* detector, std::string modelFileName, std::string textureFileName) {
    model = new Model(modelFileName, textureFileName);
    positionX = positionY = positionZ = orientation = 0;
    bshowText = false;
    textTimeLeft = 0;
    memset(textToShow, '\0', 13); 
    textColor[1] = textColor[2] = textColor[3] = 1;
    
    this->detector = detector;
    detector->addObject(this);
}

void Object::draw() {
    glPushMatrix();    
    glTranslatef(positionX,positionY,positionZ);
    glRotatef(orientation,0,1,0);
    glBindTexture(GL_TEXTURE_2D, tex);
    model->draw();
    if (bshowText) {
        glColor3f(textColor[0],textColor[1],textColor[2]);
        Utils::renderString(0,1.5,0, textToShow, GLUT_BITMAP_HELVETICA_18);    
    }
    glPopMatrix();
}

void Object::idle(float deltaTime) {
    model->idle(deltaTime);
    
    if (bshowText) {
        textTimeLeft -= deltaTime;
        if (textTimeLeft <= 0) {
            bshowText = false;    
            textTimeLeft = 0;
        }
    }
}

void Object::setPosition(float positionX, float positionY, float positionZ, float orientation) {
    this->positionX = positionX;
    this->positionY = positionY;
    this->positionZ = positionZ;
    this->orientation = orientation;
}

float Object::getPositionX() {
    return positionX;
}

float Object::getPositionY() {
    return positionY;
}

float Object::getPositionZ() {
    return positionZ;
}

float Object::getOrientation() {
    return orientation;
}

void Object::moveForward(float deltaTime) {
    float newposX = positionX + deltaTime * -cos(orientation/180.0*M_PI);
    float newposY = positionY;
    float newposZ = positionZ + deltaTime * sin(orientation/180.0*M_PI);
    if (detector->collisionDetect(this,newposX,newposY,newposZ) != NULL) {
        return;
    }
    positionX = newposX;
    positionY = newposY;
    positionZ = newposZ;
}

void Object::moveBackward(float deltaTime) {
    float newposX = positionX - deltaTime * -cos(orientation/180.0*M_PI);
    float newposY = positionY;
    float newposZ = positionZ - deltaTime * sin(orientation/180.0*M_PI);
    if (detector->collisionDetect(this,newposX,newposY,newposZ) != NULL) {
        return;
    }
    positionX = newposX;
    positionY = newposY;
    positionZ = newposZ;
}

void Object::rotateCW(float deltaTime) {
    orientation -= deltaTime*100;
}

void Object::rotateCCW(float deltaTime) {
    orientation += deltaTime*100;
}

void Object::getBoundingVolume(float *minX,float *maxX,
                                float *minY,float *maxY,
                                float *minZ,float *maxZ) {
    model->getBoundingVolume(minX,maxX,minY,maxY,minZ,maxZ);
    
    *minX = *minX;
    *maxX = *maxX;
    *minY = *minY;
    *maxY = *maxY;
    *minZ = *minZ;
    *maxZ = *maxZ;

    float x1,x2,x3,x4,z1,z2,z3,z4;
    
    x1 = *minX * cos(orientation/180.0*M_PI) + *minZ * sin(orientation/180.0*M_PI);
    x2 = *minX * cos(orientation/180.0*M_PI) + *maxZ * sin(orientation/180.0*M_PI);
    x3 = *maxX * cos(orientation/180.0*M_PI) + *minZ * sin(orientation/180.0*M_PI);
    x4 = *maxX * cos(orientation/180.0*M_PI) + *maxZ * sin(orientation/180.0*M_PI);
    
    z1 = *minX * -sin(orientation/180.0*M_PI) + *minZ * cos(orientation/180.0*M_PI);
    z2 = *minX * -sin(orientation/180.0*M_PI) + *maxZ * cos(orientation/180.0*M_PI);
    z3 = *maxX * -sin(orientation/180.0*M_PI) + *minZ * cos(orientation/180.0*M_PI);
    z4 = *maxX * -sin(orientation/180.0*M_PI) + *maxZ * cos(orientation/180.0*M_PI);
    
    *minX = min(x1,min(x2,min(x3,x4)));
    *minZ = min(z1,min(z2,min(z3,z4)));
    *maxX = max(x1,max(x2,max(x3,x4)));
    *maxZ = max(z1,max(z2,max(z3,z4)));
    
}

void Object::showText(char* text, float timePeriod) {
    bshowText = true;
    int len = strlen(text);
    memset(textToShow, '\0', 13);
    if (len < 12) {
        strncpy(textToShow, text, len);
    } else {
        strncpy(textToShow, text, 12);
    }
    textTimeLeft = timePeriod;
}
