#ifndef PARTICLE_H
#define PARTICLE_H

#include <GL/glut.h>

/*
 * Abstract Particle, subclasses must provide 
 * implementation of draw() and idle(float deltaTime)
 */
class Particle {
  protected:  
    GLfloat position[3];
    bool alive;
    
  public:
    Particle();
    Particle(GLfloat x, GLfloat y, GLfloat z);
   
    //implemented by subclasses
    virtual void draw() = 0;
    virtual void idle(float deltaTime) = 0;
    
    GLfloat* getPosition() { return position; }
    GLfloat getPosX() { return position[0]; }
    GLfloat getPosY() { return position[1]; }
    GLfloat getPosZ() { return position[2]; }
    
    void setPosX(GLfloat x) { position[0] = x; }
    void setPosY(GLfloat y) { position[1] = y; }
    void setPosZ(GLfloat z) { position[2] = z; }   
    
    bool isAlive() { return alive; }
    void setAlive(bool alive) { this->alive = alive; }
};

#endif
