#include "Room.h"
#include "Utils.h"
#include "Spider.h"
#include "Beast.h"
#include "Level.h"
#include <typeinfo>

Room::Room(Detector* detector, Wizard* wizard,
           float xmin, float xmax, float zmin, float zmax,
           int doorLocation, int wallLocation, bool bossRoom, Level* level, int wall_type) {
    this->detector = detector;
    this->wizard = wizard;
    this->level = level;
    
    this->xmin = xmin;
    this->xmax = xmax;
    this->zmin = zmin;
    this->zmax = zmax;
    
    Wall* wall;
    
    if (wallLocation & 1) {
        if (doorLocation & 1) {        
            float xl = xmin+(xmax-xmin)/2-2.5;
            wall = new Wall(detector, wall_type);
            wall->setPosition(xmin + (xl - xmin)/2 - 0.125,0,zmin,0);
            wall->setLength(xl - xmin + 0.25);
            walls.push_back(wall);
            
            float xr = xmin+(xmax-xmin)/2+2.5;
            wall = new Wall(detector, wall_type);
            wall->setPosition(xr + (xmax - xr)/2 + 0.125,0,zmin,0);
            wall->setLength(xmax - xr + 0.25);
            walls.push_back(wall);
        } else {
            wall = new Wall(detector, wall_type);
            wall->setPosition(xmin + (xmax - xmin)/2,0,zmin,0);
            wall->setLength(xmax - xmin + 0.5);
            walls.push_back(wall);
        }
    }
    
    if (wallLocation & 2) {
        if (doorLocation & 2) {
            float xl = xmin+(xmax-xmin)/2-2.5;
            wall = new Wall(detector, wall_type);
            wall->setPosition(xmin + (xl - xmin)/2 - 0.125,0,zmax,0);
            wall->setLength(xl - xmin + 0.25);
            walls.push_back(wall);
            
            float xr = xmin+(xmax-xmin)/2+2.5;
            wall = new Wall(detector, wall_type);
            wall->setPosition(xr + (xmax - xr)/2 + 0.125 ,0,zmax,0);
            wall->setLength(xmax - xr + 0.25);
            walls.push_back(wall);
        } else {
            wall = new Wall(detector, wall_type);
            wall->setPosition(xmin + (xmax - xmin)/2,0,zmax,0);
            wall->setLength(xmax - xmin + 0.5);
            walls.push_back(wall);
        }
    }
    
    if (wallLocation & 4) {
        if (doorLocation & 4) {
            float zl = zmin+(zmax-zmin)/2-2.5;
            wall = new Wall(detector, wall_type);
            wall->setPosition(xmin,0,zmin + (zl - zmin)/2,90);
            wall->setLength(zl - zmin);
            walls.push_back(wall);
            
            float zr = zmin+(zmax-zmin)/2+2.5;
            wall = new Wall(detector, wall_type);
            wall->setPosition(xmin,0,zr + (zmax - zr)/2,90);
            wall->setLength(zmax - zr);
            walls.push_back(wall);
        } else {
            wall = new Wall(detector, wall_type);
            wall->setPosition(xmin,0,zmin + (zmax - zmin)/2,90);
            wall->setLength(zmax-zmin);
            walls.push_back(wall);
        }
    }
    
    if (wallLocation & 8) {
        if (doorLocation & 8) {
            float zl = zmin+(zmax-zmin)/2-2.5;
            wall = new Wall(detector, wall_type);
            wall->setPosition(xmax,0,zmin + (zl - zmin)/2,90);
            wall->setLength(zl - zmin);
            walls.push_back(wall);
            
            float zr = zmin+(zmax-zmin)/2+2.5;
            wall = new Wall(detector, wall_type);
            wall->setPosition(xmax,0,zr + (zmax - zr)/2,90);
            wall->setLength(zmax - zr);
            walls.push_back(wall);
        } else {
            wall = new Wall(detector, wall_type);
            wall->setPosition(xmax,0,zmin + (zmax - zmin)/2,90);
            wall->setLength(zmax-zmin);
            walls.push_back(wall);
        }
    }
    
    if (!bossRoom) {
        Beast* beast;
        if (rand() % 2 == 1) {
            beast = new Beast(detector, wizard, this,Utils::randomInt(1,5));
            beast->setPosition(xmin + (xmax - xmin)/2,0,zmin + (zmax - zmin)/2,0);
            enemies.push_back(beast);
        }
        if (rand() % 2 == 1) {
            beast = new Beast(detector, wizard, this,Utils::randomInt(1,5));
            beast->setPosition(xmin + (xmax - xmin)/3,0,zmin + (zmax - zmin)/3,0);
            enemies.push_back(beast);
        }
        if (rand() % 2 == 1) {
            beast = new Beast(detector, wizard, this,Utils::randomInt(1,5));
            beast->setPosition(xmin + 2*(xmax - xmin)/3,0,zmin + 2*(zmax - zmin)/3,0);
            enemies.push_back(beast);
        }
        
        Gold* gold;
        if (rand() % 2 == 1) {
            gold = new Gold(detector,this);
            gold->setPosition(xmin + 2, 0, zmin + 2, 0);
            coins.push_back(gold);
        }
        if (rand() % 2 == 1) {
            gold = new Gold(detector,this);
            gold->setPosition(xmax - 2, 0, zmax - 2, 0);
            coins.push_back(gold);
        }
        if (rand() % 2 == 1) {
            gold = new Gold(detector,this);
            gold->setPosition(xmin + 2, 0, zmax - 2, 0);
            coins.push_back(gold);
        }
        if (rand() % 2 == 1) {
            gold = new Gold(detector,this);
            gold->setPosition(xmax - 2, 0, zmin + 2, 0);
            coins.push_back(gold);
        }
    } else {
        Enemy* beast;
        beast = new Spider(detector, wizard, this,6,level);
        beast->setPosition(xmin + 2,0,zmin + (zmax - zmin)/2,0);
        enemies.push_back(beast);
    }
    
}

Room::~Room() {
    std::list<Wall*>::iterator i;
    std::list<Enemy*>::iterator j;
    std::list<Gold*>::iterator k;
    for (i = walls.begin(); i != walls.end(); i++) {
        Wall* obj = *i;
        i--;
        walls.remove(obj);
        delete obj;
    }
    for (j = enemies.begin(); j != enemies.end(); j++) {        
        Beast* obj = (Beast*) *j;
        j--;
        enemies.remove(obj);
        delete obj;
    }
    for (k = coins.begin(); k != coins.end(); k++) {
        Gold* obj = *k;
        k--;
        coins.remove(obj);
        delete obj;
    }
}

void Room::draw() {
    std::list<Wall*>::iterator i;
    std::list<Enemy*>::iterator j;
    std::list<Gold*>::iterator k;
    for (i = walls.begin(); i != walls.end(); i++) {
        Wall* obj = *i;
        obj->draw();
    }

    for (j = enemies.begin(); j != enemies.end(); j++) {
        Enemy* obj = *j;
        obj->draw();
    }
    for (k = coins.begin(); k != coins.end(); k++) {
        Gold* obj = *k;
        obj->draw();
    }
}

void Room::drawForPick() {
    std::list<Wall*>::iterator i;
    for (i = walls.begin(); i != walls.end(); i++) {
        Wall* obj = *i;
        obj->draw();
    }
    
    std::list<Enemy*>::iterator j;
    for (j = enemies.begin(); j != enemies.end(); j++) {
        Enemy* obj = *j;
        glPushName(obj->getId());
        obj->draw();
        glPopName();
    }
}

void Room::idle(float deltaTime) {
    std::list<Wall*>::iterator i;
    std::list<Enemy*>::iterator j;
    for (i = walls.begin(); i != walls.end(); i++) {
        Wall* obj = *i;
        obj->idle(deltaTime);
    }
    for (j = enemies.begin(); j != enemies.end();) {
        Enemy* obj = *j;
        if (!obj->isAlive() && obj->isDeathAnimationDone()) {
            j++;
            if (typeid(*obj) == typeid(Spider)) {
                delete ((Spider*) obj);
            } else {
                delete ((Beast*) obj);
            }
        } else {
            obj->idle(deltaTime);
            j++;
        }
    }
}

bool Room::inRoom(float x, float y, float z) {
    if (xmin < x && x < xmax && zmin < z && z < zmax) {
        return true;
    } else {
        return false;
    }
}

void Room::removeGold(Gold* object) {
    coins.remove(object);
}

void Room::removeEnemy(Enemy* object) {
    enemies.remove(object);
}
