#include "Spider.h"
#include <math.h>

Spider::Spider(Detector* detector, Wizard* wizard, Room* room, int level, Level* lev) : Enemy(detector, wizard, room, level, "Models/spider.obj", "Textures/spider.tga") {
    frame0 = new Frame("Models/spider.obj", 0.2);
    frame1 = new Frame("Models/spider-walk1.obj", 0.2);
    frame2 = new Frame("Models/spider-walk2.obj", 0.2);
    walkAnimation = new Animation(model, true);
    idleAnimation = new Animation(model, false);
    walkAnimation->addFrame(frame1);
    walkAnimation->addFrame(frame0);
    walkAnimation->addFrame(frame2);
    walkAnimation->addFrame(frame0);
    idleAnimation->addFrame(frame0);
    model->addAnimation(SPIDER_WALK, walkAnimation);
    model->addAnimation(SPIDER_REST, idleAnimation);
    model->setCurrentAnimation(SPIDER_REST);  
    model->setScale(1);
    model->setHeadingCorrection(90);
    this->lev = lev;
}

Spider::~Spider() {
    delete walkAnimation;
    delete idleAnimation;
    delete frame0;
    delete frame1;
    delete frame2;
    lev->nextLevel();
}

void Spider::moveForward(float deltaTime) {
    Object::moveForward(level*deltaTime);
    if (model->isAnimating() && (model->getCurrentAnimation() != SPIDER_WALK)) {
        model->setCurrentAnimation(SPIDER_WALK);
    } else if (!model->isAnimating()) {
        model->setCurrentAnimation(SPIDER_WALK);
        model->startAnimation();
    }
}

void Spider::stop() {
    model->setCurrentAnimation(SPIDER_REST);
    if (!model->isAnimating()) {
        model->startAnimation();    
    }
}
